#!/bin/bash
#
# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018, 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************
#

if [ "${LIB_RANKFILE_SOURCED}" = "" ] ; then
LIB_RANKFILE_SOURCED="true"

##############################################################################
# If a rankfile is being used, check to see if it exists.
# Globals:
#   None
# Arguments:
#   None
# Returns:
#   None
##############################################################################
rankfileTest()
{
    local RANK_FILE=${1}
    if [ "${RANK_FILE}" != "" ] && [ ! -f "${RANK_FILE}" ] ; then
        print_error "ERROR" $MSG_NO_RANKFILE
        exit 1
    fi
}

##############################################################################
# Uses the specified rank gen arguments to generate a rankfile in the specified
# location.
# Globals:
#   ${VERBOSE}
#   ${WORKING_DIR}
#   ${MPI_ARGS}
# Arguments:
#   Host List
#   Rank file location
#   Additional rankgen arguments
# Returns:
#   None
##############################################################################
generateRankFile()
{
    local HOSTS="${1}"
    shift
    local RANKFILE="${1}"
    shift
    local SPLIT_HOSTS=($(echo ${HOSTS} | tr "," " "))
    snapml_rank_gen.py "-w ${WORKING_DIR}" ${HOSTS} > "${RANKFILE}" 2> "${WORKING_DIR}/rankgen.err"
    local ret_code=$?
    if [ ${ret_code} -ne 0 ]; then
        print_error "ERROR" $MSG_RANK_FILE_ERROR
        [ "${VERBOSE}" = "true" ] && >&2 cat "${WORKING_DIR}/rankgen.err"
        exit ${ret_code}
    fi
}

##############################################################################
# Parse the host file and handle errors.
# Globals:
#   None
# Arguments:
#   Hostfile location
# Returns:
#   Comma delimited host list.
##############################################################################
parseHostFile()
{
  local HOST_FILE=${1}
  if [ ! -f "${HOST_FILE}" ] ; then
    print_error "ERROR" $MSG_NO_HOSTFILE
    exit 1
  fi

  local HOST_FILE_CONTENTS=$(cat "${HOST_FILE}")

  # Ignore comments.
  HOST_FILE_CONTENTS=$(echo "${HOST_FILE_CONTENTS}" | sed 's/#.*$//g')

  # Trim whitespace.
  HOST_FILE_CONTENTS=$(echo "${HOST_FILE_CONTENTS}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')

  #Check for lines with commas/spaces to give better error message.
  if echo "${HOST_FILE_CONTENTS}" | grep -Eq "( |,)" ; then
    print_error "ERROR" $MSG_HOSTFILE_COMMA_SPACE
    exit 1
  fi

  # Delimit lines with commas.
  HOST_FILE_CONTENTS=$(echo "${HOST_FILE_CONTENTS}" | paste -sd ",")
  
  # Ignore empty lines.
  HOST_FILE_CONTENTS=$(echo "${HOST_FILE_CONTENTS}" | sed -e 's/,\+/,/g' -e 's/^,//g' -e 's/,$//g')

  echo "${HOST_FILE_CONTENTS}"
}


fi
