# Credit Card Fraud Example

In this example we will train a Logistic Regression model on a credit card fraud dataset, using both `scikit-learn` and `pai4sk`. 
Note that `scikit-learn` does not provide an MPI implementation. We will however still perform this comparison in order to mainly verify the performance scores.

The creditcard dataset is avaliable in the [Kaggle credit card fraud detection competition](https://www.kaggle.com/mlg-ulb/creditcardfraud/data).

## Download the Data

Download and decompress the data from the LIBSVM repository:
```
mkdir data
cd data
```

Click on `Download` in the page https://www.kaggle.com/mlg-ulb/creditcardfraud/data and copy the data zip file into the data directory.
```
unzip creditcardfraud.zip
cd ../
```

Now one should see a file named `creditcard.csv` in the `data` directory.

## Data Pre-processing
The data is in libsvm format. We start by performing some pre-processing.
The entire pre-processing script can be run by executing `snaprun python $CONDA_PREFIX/pai4sk/mpi-examples/example-creditcard/preprocess-creditcard.py`. 

A summary of all the pre-processed files is shown below:
```bash
ls -l data/
creditcard.csv
creditcard.augmented.test.snap
creditcard.augmented.train.snap
creditcard.augmented.X_test.npy
creditcard.augmented.X_train.npy
creditcard.augmented.y_test.npy
creditcard.augmented.y_train.npy
```

## Training and Evaluating a Logistic Regression Model

After running the data pre-processing step, we are ready to perform our benchmarking: `snap.ml` vs. `sklearn`.

First we train a Logistic Regression model using the IBM `pai4sk` library. 
We provide the command line option --use_gpu to run the example for 1) GPU or 2) CPU training.

We define the data paths for the train and test datasets in distributed snap format.
You need to have access to the data from all the connected machines that will be used (by replicating the data or by providing a distributed file system).

Next we read the datasets from snap format using the loading functions provided by the `pai4sk` library and we measure the data loading time.

After model initialization, we call the `fit()` function in order to train the Logistic Regression model. We also time the execution of the training process.

Please note that the `pai4sk` library assumes for maximum performance that it runs 1 MPI process per node.

As a last step we make predictions on the test data and we compute the accuracy score. 
The `pai4sk` library provides a distributed implementation of the accuracy metric. 

Finally, we train a Logistic Regression model using the `sklearn` library.

## Results

To run on one machine with 1 MPI process, one should run the following command: 
```bash
snaprun python $CONDA_PREFIX/pai4sk/mpi-examples/example-creditcard/example-creditcard.py
```

To run on 2 machines with 1 MPI process per machine, one should run the following command:
```bash
snaprun -H host1,host2 python $CONDA_PREFIX/pai4sk/mpi-examples/example-creditcard/example-creditcard.py
```

The user can also run multiple MPI processes per machine, but, in that case, the MPI processes on one machine will share the same hardware resources during training. Namely, if the user trains the model on GPUs, the MPI processes on one machine will share the same GPUs as defined in the `device_ids` parameter. Therefore, we recommend the user to run 1 MPI process per machine in order to achieve the maximum performance.

One should see the following output on a **P9+V100** system when running
`snaprun python $CONDA_PREFIX/pai4sk/mpi-examples/example-creditcard/example-creditcard.py`:

```
[pai4sk] [Data loading] Time = 1.589
[pai4sk] [GPU Training] Accuracy = 0.853
[pai4sk] [GPU Training] Training time = 3.471

[sklearn] [Data loading] Time = 1.687
[sklearn] Accuracy = 0.853
[sklearn] Training time = 177.831
```

One should see the following results on 2 **P9+V100** systems when running `snaprun -H host1,host2 python $CONDA_PREFIX/pai4sk/mpi-examples/example-creditcard/example-creditcard.py`:

```
[pai4sk] [Data loading] Time = 2.430
[pai4sk] [GPU Training] Accuracy = 0.853
[pai4sk] [GPU Training] Training time = 3.220

[sklearn] [Data loading] Time = 1.687
[sklearn] Accuracy = 0.853
[sklearn] Training time = 177.831
```

Please note that the reported times can vary depending on the system architecture.

&copy; Copyright IBM Corporation 2018, 2019

